﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2011.03.21.
 * Idő: 10:14
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.Xml;
//
using TetheredSun;

namespace Noise.DataAcquisition
{
	/// <summary>
	/// Description of SensorInterfacing.
	/// </summary>
	public class SensorInterfacing : IXmlConvertible
	{
		protected int id;
		protected string label;
		protected static readonly string[] labels = new string[] {
			"Voltage output",
			"Resistance output",
			"Internal photosensor",
			"Voltage difference (B-C)",
			"Wheatstone bridge (B-C)"
		};
		
		public SensorInterfacing(int id)
		{
			this.id = id;
			if (this.id < 0) this.id = 0;
			if (this.id > 4) this.id = 0;
			label = labels[this.id];
		}
		
		public int Id {
			get { return id; }
		}
		
		public string Label {
			get { return label; }
		}
		
		
		public static SensorInterfacing VoltageOutput {
			get { return new SensorInterfacing(0); }
		}
		
		public static SensorInterfacing ResistanceOutput {
			get { return new SensorInterfacing(1); }
		}
		
		public static SensorInterfacing InternalPhotosensor {
			get { return new SensorInterfacing(2); }
		}
		
		public static SensorInterfacing Differential {
			get { return new SensorInterfacing(3); }
		}
		
		public static SensorInterfacing Wheatstone {
			get { return new SensorInterfacing(4); }
		}
		
		public bool CheckIfEnabled(string channelName)
		{
			if (id < 0) return false;
			if (id > 4) return false;
			switch (channelName.ToUpper()) {
				case "A":
					if (id < 3)
						return true;
					else
						return false;
				case "B":
					if (id == 2)
						return false;
					else
						return true;
				case "C":
					if (id < 2)
						return true;
					else
						return false;
				default:
					return false;
			}
		}
		
		public void FromXml(XmlElement xmlElement)
		{
			if (xmlElement.Name != "SensorInterfacing") {
				Exception exception = new ArgumentException("Invalid element name.");
				
				exception.Data.Add("Expected name", "SensorInterfacing");
				exception.Data.Add("Received name", xmlElement.Name);
				
				throw exception;
			}
			
			int parsedInt;
			
			if (Int32.TryParse(xmlElement.InnerXml, out parsedInt)) {
				if (parsedInt < 0) parsedInt = 0;
				if (parsedInt > 4) parsedInt = 4;
				this.id = parsedInt;
				this.label = labels[this.id];
			}
		}
		
		public override string ToString()
		{
			return label;
		}
		
		public XmlElement ToXml(XmlDocument context)
		{
			XmlElement sensorInterfacingXml = context.CreateElement("SensorInterfacing");
			
			sensorInterfacingXml.SetAttribute("label", label);
			sensorInterfacingXml.InnerXml = id.ToString();
			
			return sensorInterfacingXml;
		}
		
	}
}
