﻿/*
 * Created by SharpDevelop.
 * User: phil
 * Date: 2010.09.23.
 * Time: 14:28
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Xml;
//
using TetheredSun;

namespace TetheredSun.SignalAnalysis
{
	/// <summary>
	/// Description of Sine.
	/// </summary>
	public class Sine : IXmlConvertible
	{
		protected double frequency;
		protected double amplitude;
		protected double phase;
		
		public Sine(double frequency, double amplitude, double phase)
		{
			this.frequency = frequency;
			this.amplitude = amplitude;
			this.phase = phase;
		}
		
		public Sine() : this(1.0, 1.0, 0.0) { }
		
		public double Frequency {
			get { return frequency; }
			set { frequency = value; }
		}
		
		public double Amplitude {
			get { return amplitude; }
			set { amplitude = value; }
		}
		
		public double Phase {
			get { return phase; }
			set { phase = value; }
		}
		
		public void FromXml(XmlElement xmlElement)
		{
			if (xmlElement.Name != "Sine") {
				Exception exception = new ArgumentException("Invalid element name.");
				
				exception.Data.Add("Expected name", "Sine");
				exception.Data.Add("Received name", xmlElement.Name);
				
				amplitude = Double.NaN;
				frequency = Double.NaN;
				phase = Double.NaN;
				
				throw exception;
			}
			
			double parsedDouble;
			XmlNode child;
			
			child = xmlElement.SelectSingleNode("child::Amplitude");
			amplitude = RobustParser.TryParse(child.InnerXml, out parsedDouble) ? parsedDouble : Double.NaN;
			
			child = xmlElement.SelectSingleNode("child::Frequency");
			frequency = RobustParser.TryParse(child.InnerXml, out parsedDouble) ? parsedDouble : Double.NaN;
			
			child = xmlElement.SelectSingleNode("child::Phase");
			phase = RobustParser.TryParse(child.InnerXml, out parsedDouble) ? parsedDouble : Double.NaN;
		}	
		
		public double GetValue(double time)
		{
			return amplitude * Math.Sin(2.0 * Math.PI * frequency * time + phase);
		}
		
		public double[] GetValues(double[] timeInstances)
		{
			double[] values = new double[timeInstances.Length];
			
			for (int i = 0; i < values.Length; i++) {
				values[i] = GetValue(timeInstances[i]);
			}
			
			return values;
		}
		
		public double[] GetValues(double t0, double dt, int count)
		{
			double[] values = new double[count];
			double t = t0;
			
			for (int i = 0; i < values.Length; i++) {
				t += i * dt;
				values[i] = GetValue(t);
			}
			
			return values;
		}
		
		public override string ToString()
		{
			return String.Format("Amplitude: {0}. Frequency: {1}. Phase: {2}.", amplitude, frequency, phase);
		}
		
		public XmlElement ToXml(XmlDocument context)
		{
			XmlElement sineXml = context.CreateElement("Sine");
			XmlElement child;
			
			child = context.CreateElement("Amplitude");
			child.InnerXml = amplitude.ToString();
			sineXml.AppendChild(child);
			
			child = context.CreateElement("Frequency");
			child.SetAttribute("unit", "Hz");
			child.InnerXml = frequency.ToString();
			sineXml.AppendChild(child);
			
			child = context.CreateElement("Phase");
			child.SetAttribute("unit", "rad");
			child.InnerXml = phase.ToString();
			sineXml.AppendChild(child);
			
			return sineXml;
		}
		
		public static bool TryParse(string text, out Sine sine)
		{
			// Assumed format: amplitude; frequency; phase:
			string[] split = text.Split(new string[] { ";" }, StringSplitOptions.RemoveEmptyEntries);
			double a;
			double f;
			double p;
			
			if (split.Length < 1) {
				sine = null;
				return false;
			}
			
			if (RobustParser.TryParse(split[0].Trim(), out a)) {
				if (RobustParser.TryParse(split[1].Trim(), out f)) {
					if (RobustParser.TryParse(split[2].Trim(), out p)) {
						sine = new Sine(f, a, p);
						return true;
					} else {
						sine = new Sine(f, a, 0.0);
						return true;
					}
				}
			}
			
			sine = null;
			return false;
		}
	}
}
